      Subroutine chol(a,l,n,ifail)
c
c  SCOPO: fattorizzazione di Cholesky di una matrice A simmetrica
c         definita positiva nella forma A=L*L', con L matrice
c         triangolare inferiore ed L' sua trasposta.
c 
c  PARAMETRI DI INPUT:  
c 
c  - a,  vettore di reali,  elementi del triangolo inferiore di A
c  - n,            intero,  ordine della matrice
c 
c  PARAMETRI DI OUTPUT:
c
c  -     l,  vettore di reali,  elementi del triangolo inferiore di L
c  - ifail,            intero,  indicatore di errore
c   
c  ROUTINES RICHIAMATE:
c
c  - nessuna
c
       integer n,ifail
       real a(n*(n+1)/2),l(n*(n+1)/2)

c  Controllo della dimensione      
       if (n .le. 0) then
          ifail=3
          goto 999
       endif

c  Fattorizzazione di Cholesky       
       do j=1,n
          
          id=j*(j-1)/2+j
          l(id)=a(id)
          do k=1,j-1
             l(id)=l(id)-l(j*(j-1)/2+k)**2      
          end do

c     Controlla che la matrice sia definita positiva
          if (l(id) .lt. 0) then
             ifail=5
             goto 999
          endif
          
          l(id)=sqrt(l(id))
          
          do i=j+1,n
             il=i*(i-1)/2+j
             l(il)=a(il)
             do k=1,j-1
                l(il)=l(il)-l(i*(i-1)/2+k)*l(j*(j-1)/2+k)
             end do
             l(il)=l(il)/l(id)    
          end do
           
       end do
          
       ifail=0
                
  999  continue
  
      End
	stop