      Subroutine corit(lu,p,b,x,y,ldlu,n,tol,maxit,r,ifail,log)

c
c     BETA 1.5
c

c
c  SCOPO: correzione iterativa della soluzione di un sistema li-
c         neare A*x=b mediante il sistema equivalente (L*U)*x=P*b,
c         dove P  una matrice di permutazione ed L*U la fattoriz-
c         zazione di P*A fornita in uscita dalla subroutine "lup".
c
c  PARAMETRI DI INPUT: 
c                                                   
c  -    lu,   matrice di reali,  permutate le righe secondo p
c                                restituisce i fattori L ed U 
c  -     p,  vettore di interi,  indici delle righe di a da 
c                                permutare per ottenere L ed U   
c  -     b,   vettore di reali,  vettore termini noti
c  -     x,   vettore di reali,  soluzione iniziale
c  -     y,   vettore di reali,  ricostruisce Ax attraverso il prodotto L*U*x

c  -     r,   vettore di reali,  residuo del sistema (doppia precisione)
c  -   tol,              reale,  tolleranza richiesta
c  - maxit,             intero,  massimo numero di iterazioni
c  -  ldlu,             intero,  leading dimension di a dichiarata
c                                nel programma chiamante
c  -     n,             intero,  ordine del sistema
c
c  PARAMETRI DI OUTPUT:
c
c  - ifail,            intero,  indicatore di errore
c
c  ROUTINE RICHIAMATE:
c
c  - nessuna
c
              
       integer maxit,ldlu,n,p(n),ifail,iter,log
       
       real lu(ldlu,n), b(n),y(n), x(n), norma, tol
       
       double precision r(n)

c     controllo della dimensione
      if (n .le. 0 .or. n .gt. ldlu) then
          ifail = 3
          goto 999
      endif
      
c     controllo della tolleranza
      if (tol .lt. 0) then
          ifail = 1
          goto 999
      endif

c     Controllo sul numero minimo di iterazioni richieste
      if (maxit .lt. 1) then
          ifail = 8
          goto 999
      endif                                    

c     Controllo della singolarit di lu
      do 5 i=1,n
          if (lu(p(i),i) .eq. 0) then 
              ifail = 4
              goto 999
          endif
    5 continue   
    
c     s=iter                                   
      iter=0   
        
c     inizializzazione di r
      do i=1,n
          r(i)=0.0
      end do

   10 continue   
      
      iter=iter+1  
      
      write(log,*)'passo: ',iter    
      write(log,*)'risoluzione r= Ax - b'  
      write(log,*)
      
c                     (s)        (s)                   
c     Risoluzione di r     =    Ax   -  b
c     in input non viene fornita A, bens la sua fattorizzazione LU
c     quindi per poter effettuare il prodotto Ax,  necessario ricostruire A
c     PA=LU => Ax = P(LU)x = P(L(Ux)) => r = P(L(Ux)) - b
c     utilizzo come variabile di appoggio y

C     calcolo di Ux
      write(log,*)'calcolo di Ux'
      do i=1,n    
          y(i)=0.0
          do j=i,n
              y(i)=y(i)+lu(p(i),j)*x(j) 
          end do
          write(log,*)y(i)
      end do
                                                 
C     calcolo di Ly, (y=Ux)
c     il ciclo su j si ferma ad i-1, perch nella fattorizzazione lu, L nella diagonale
c     principale, contiene elementi tutti uguali ad 1, se moltiplico per l'iesimo 
c     elemento, data l'organizzazione della struttura dati, moltiplico per elementi
c     appartenenti alla diagonale principale di U e non di L
      write(log,*)'calcolo di Ly'
      do i=1,n
          do j=1,i-1
              y(i)=y(i)+lu(p(i),j)*y(j)
          end do
          write(log,*)y(i)
      end do
C     y contiene il prodotto Ax                                

C     calcolo di r = Ax - b
      write(log,*)'calcolo di r = Ax-b'
      do i=1,n
          r(i) = y(i) - b(i)         
          write(log,*)'r(i)= ',r(i)
      end do
c
c     da questo punto in poi, y non viene piu' utilizzata
c     quindi la posso riusare per calcolare Adx=r  
c
c      inizializzazione di y
       do i=1,n
       	    y(i)=0.0
       end do

c     Risoluzione di Adx=r, effettuata applicando
c     la forward e la backward substitution, 
c     implementate per colonna; A*dx = r ==> L*(U*dx)=P*r,  U*dx = y

c      Risoluzione di L*y=P*r   
       write(log,*)
       write(log,*)'Ly=Pr'


       do 40 j=1,n-1  
c          r(j)=r(j)/lu(p(j),j)
	  
          do 35 i=j+1,n
             y(i)=y(i)+lu(p(i),j)*r(j)                              
   35     continue   
          write(log,*)'r(',j,')=',y(j)
   40  continue              
      y(n)=y(n)/lu(p(n),n)

c     Risoluzione di U*x=y
       write(log,*)'Ux=y'

       do 50 j=n,2,-1
          y(j)=y(j)/lu(p(j),j)
          do 45 i=j-1,1,-1             
             y(i)=y(i)-lu(p(i),j)*y(j) 
   45     continue       
          write(log,*)'r(',j,')=',y(j)
   50  continue   
       y(1)=y(1)/lu(p(1),1)    

c
c     verificatori
c       
       write(log,*)
       write(log,*)'dx calcolato:'
       do i=1,n
          write(log,*)'dx(',i,')= ',y(i)
       end do
       write(log,*)
       write(log,*)'soluzione al passo: ',iter
       do i=1,n
          write(log,*)'x(',i,')= ',x(i)
       end do

c                  (s+1)   (s)
c      dx = r,    x     = x   + dx        
c
   
C     Calcolo della soluzione approssimante  
      write(log,*)
      write(log,*)'calcolo della sol. approssimante al passo',iter+1
      do 60 i=1,n       
          x(i)=x(i)+y(i)  
          write(log,*)'x(',iter+1,')(i)= ',x(i)          
   60 continue                                  
   
      write(log,*)
      write(log,*)'residuo:'
      do i=1,n
          write(log,*)'r(',i,') ',r(i)
      end do
   
C     Calcolo della norma infinita di dx(=r) , ||dx|| < tol

      norma=0        
      write(log,*)'norma= ',norma
      do 70 i=1,n
          if (abs(y(i)) .gt. norma) then
              norma = abs(y(i))
          endif
   70 continue

c     controllo sul numero di iterazioni effettuate
      if (iter .eq. maxit) then
          ifail = 9
          goto 999
      endif

C     controllo sul raggiungimento delle condizioni di uscita dal ciclo   

      if (norma .lt. tol) goto 10
            
  999 continue   
      end
       