      Subroutine pmq1(x,y,n,c,r,ifail)
c
c  SCOPO: determinazione dei coefficienti del polinomio dei minimi
c         quadrati di 1 grado.
c 
c  PARAMETRI DI INPUT:  
c 
c  - x,  vettore di reali,  ascisse dei punti
c  - y,  vettore di reali,  ordinate dei punti
c  - n,            intero,  numero di punti
c 
c  PARAMETRI DI OUTPUT:
c
c  -     c,  vettore di reali,  coefficienti del polinomio
c  -     r,  vettore di reali,  vettore dei residui
c  - ifail,            intero,  indicatore di errore
c   
c  ROUTINES RICHIAMATE:
c
c  - chol,fsbpk,bsbpk
c  
       external chol,fsbpk,bsbpk
       
       integer n,ifail          
       real x(n),y(n),c(2),r(n)
   
       real b(3),d(2),l(3)
       
       ifail=0
                    
c  Controllo del numero di punti
       if (n .le. 0) then
          ifail=11
          goto 999
       endif
       if (n .eq. 1) then
          ifail=13
          goto 999
       endif
       if (n .eq. 2) then ifail=14
       
c  Controllo delle ascisse
       do 2 i=2,n
         if (x(1) .ne. x(i)) then goto 2
         else          
       		ifail=13
       		goto 999
         endif
     2 continue

c  Calcolo dei coefficienti e dei termini noti
c  del sistema delle equazioni normali
       b(1)=n
       b(2)=0
       do 10 i=1,n
          b(2)=b(2)+x(i)
   10  continue
       b(3)=0
       do 20 i=1,n
          b(3)=b(3)+x(i)**2
   20  continue  
   
       c(1)=0
       do 30 i=1,n
          c(1)=c(1)+y(i)
   30  continue
       c(2)=0 
       do 40 i=1,n
          c(2)=c(2)+x(i)*y(i)
   40  continue   
   
c  Fattorizzazione di Cholesky della matrice dei coefficienti
       call chol(b,l,2,ifail)
       
c  Risoluzione del sistema  
       call fsbpk(l,c,2,ifail)
       call bsbpk(l,c,2,ifail)

c  Calcolo dei residui
       do 50 i=1,n
          r(i)=y(i)-(c(1)+c(2)*x(i))
   50  continue                   

  999  continue
  	
	stop
       End