      Subroutine pmq2(x,y,n,c,r,ifail)
c
c  SCOPO: calcolo dei coefficienti del polinomio dei minimi quadrati
c         di secondo grado.
c 
c  PARAMETRI DI INPUT:  
c 
c  - x,  vettore di reali,  ascisse dei punti
c  - y,  vettore di reali,  ordinate dei punti
c  - n,            intero,  numero dei punti
c 
c  PARAMETRI DI OUTPUT:
c
c  -     c,  vettore di reali,  coefficienti
c  -     r,  vettore di reali,  vettore dei residui
c  - ifail,            intero,  indicatore di errore
c   
c  ROUTINES RICHIAMATE:
c
c  - chol,fsbpk,bsbpk
c  
       external chol,fsbpk,bsbpk
       
       integer ifail          
       real x(n),y(n),c(3),r(n)

       real b(6),l(6)
       
       ifail=0
                    
c  Controllo del numero di punti
       if (n .le. 0) then
          ifail=11
          goto 999
       endif 
       if (n .le. 2) then
          ifail=13
          goto 999
       endif      
       if (n .eq. 3) ifail=14

c  Controllo delle ascisse
       do 1 i=2,n
          if (x(1) .ne. x(i)) goto 2
    1  continue
       ifail=13
       goto 999
    2  continue

c  Inizializzazione del vettore r(i)
       do 10 i=1,n
          r(i)=x(i)
   10  continue
    
c  Calcolo dei coefficienti e dei termini noti del sistema
c  di equazioni normali
       b(1)=n
       
       c(1)=0
       do 20 i=1,n
          c(1)=c(1)+y(i)
   20  continue
   
       b(2)=0
       c(2)=0
       do 30 i=1,n
          b(2)=b(2)+r(i)
          c(2)=c(2)+r(i)*y(i)
   30  continue
          
       b(3)=0
       do 40 i=1,n
          r(i)=r(i)**2
          b(3)=b(3)+r(i)
          c(3)=c(3)+r(i)*y(i)
   40  continue     
       
       b(4)=b(3)

       do 60 k=5,6
          b(k)=0
          do 50 i=1,n
             r(i)=r(i)*x(i)
             b(k)=b(k)+r(i)
   50     continue        
   60  continue
           
c  Fattorizzazione di Cholesky della matrice dei coefficienti b=l*l'
       call chol(b,l,3,ifail)
   
c  Risoluzione del sistema l*w=d
       call fsbpk(l,c,3,ifail)
              
c  Trasposta della matrice l
       swap=l(3)
       l(3)=l(4)
       l(4)=swap
       
c  Risoluzione del sistema l'*c=w
       call bsbpk(l,c,3,ifail)
       
c  Calcolo dei residui
       do 70 i=1,n
          r(i)=y(i)-(c(1)+c(2)*x(i)+c(3)*x(i)**2)
   70  continue       
       
  999  continue
  
      End