      Subroutine splin(p,a,b,c,d,n,m,x,y,ifail)
c                
c  SCOPO: valutazione della spline cubica naturale di assegnati
c         coefficienti.
c
c  PARAMETRI DI INPUT:
c
c  - p,  vettore di reali,  ascisse dei punti di interpolazione
c  - a,  vettore di reali,  coefficienti dei termini di grado 3
c  - b,  vettore di reali,  coefficienti dei termini di grado 2
c  - c,  vettore di reali,  coefficienti dei termini di grado 1
c  - d,  vettore di reali,  coefficienti dei termini di grado 0
c  - n,            intero,  numero di punti di interpolazione
c  - m,            intero,  numero di punti in cui valutare la
c                           spline
c  - x,  vettore di reali,  punti in cui valutare la spline
c
c  PARAMETRI DI OUTPUT:
c
c  -     y,  vettore di reali,  valori della spline nei punti di x
c  - ifail,            intero,  indicatore d'errore
c
c  ROUTINE RICHIAMATE:
c
c  - nessuna
c
       integer n,ifail
       real p(n),a(n-1),b(n-1),c(n-1),d(n-1),x(m),y(m)
       
       logical ord               
       integer primo,ultimo
                      
c  Controllo del numero di punti
        if (n .lt. 3) then
           ifail=11
           goto 999
        endif
        
c  Ordinamento delle ascisse
       k=n
   10  continue
          ord=.true.
          do 20 i=1,k-1   
             if (p(i) .gt. p(i+1)) then
                t=p(i)
                p(i)=p(i+1)
                p(i+1)=t
                ord=.false.
             endif  
   20     continue
          k=k-1   
       if (.not. ord) goto 10

c  Controllo dell'assenza di ascisse uguali
       do 25 i=1,n-1
          if (x(i) .eq. x(i+1)) then
             ifail=12
             goto 999
          endif
   25  continue

c  Controllo dell'appartenenza dei punti di x all'intervallo [p(1),p(n)]
       do 30 i=1,m
          if (x(i) .lt. p(1)  .or.  x(i) .gt. p(n)) then
             ifail=15
             goto 999
          endif
   30  continue
         
       do 60 j=1,m
c     Ricerca binaria di x(j) in p
          primo=1
          ultimo=n     
   40     continue
          i=(primo+ultimo)/2
          if (x(j) .ge. p(i)  .and.  x(j) .le. p(i+1)) goto 50
             if (x(j) .lt. p(i)) ultimo=i
             if (x(j) .gt. p(i+1)) primo=i+1
          goto 40
   50     continue
   
c     Valutazione della spline in x(j)
          h=x(j)-p(i)
          y(j)=((a(i)*h+b(i))*h+c(i))*h+d(i)
   
   60  continue
       
       ifail=0
     
  999  continue
  
      End
      stop
